import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import Quagga from '@ericblade/quagga2';
import { Subject } from 'rxjs';

class NgxBarcodeScannerService {
    constructor() {
    }
    defaultConfig() {
        return {
            inputStream: {
                name: 'Live',
                type: 'LiveStream',
            },
            locator: {
                patchSize: 'medium',
                halfSample: false
            },
            locate: true,
            numOfWorkers: 8,
            frequency: 10
        };
    }
    meanBy(arr, property) {
        if (!arr) {
            return undefined;
        }
        return arr.reduce((acc, item) => (property in item) ? acc + item[property] : acc, 0) / arr.length;
    }
    isScanMatch(scanResult, errorThresholdPercentage) {
        const avgErrors = this.meanBy(scanResult.codeResult.decodedCodes, 'error');
        return avgErrors < errorThresholdPercentage;
    }
    start(config, errorThresholdPercentage) {
        if (typeof this.scanResult === 'undefined') {
            this.scanResult = new Subject();
        }
        Quagga.onProcessed((scanResult) => {
            this.onProcessed(scanResult);
        });
        Quagga.onDetected((result) => {
            const barcode = result.codeResult.code;
            if (this.isScanMatch(result, errorThresholdPercentage)) {
                this.scanResult.next(barcode + '');
            }
        });
        Quagga.init(config, (error) => {
            if (error) {
                this.scanResult.error(error);
                this.stop();
            }
            else {
                Quagga.start();
            }
        });
        return this.scanResult;
    }
    stop() {
        if (typeof this.scanResult !== 'undefined') {
            this.scanResult.unsubscribe();
        }
        Quagga.stop();
    }
    onProcessed(result) {
        const drawingCtx = Quagga.canvas.ctx.overlay;
        const drawingCanvas = Quagga.canvas.dom.overlay;
        if (result) {
            if (result.boxes) {
                drawingCtx.clearRect(0, 0, parseInt(drawingCanvas.getAttribute('width'), 10), parseInt(drawingCanvas.getAttribute('height'), 10));
                result.boxes.filter((box) => box !== result.box).forEach((box) => {
                    Quagga.ImageDebug.drawPath(box, { x: 0, y: 1 }, drawingCtx, { color: 'green', lineWidth: 2 });
                });
            }
            if (result.box) {
                Quagga.ImageDebug.drawPath(result.box, { x: 0, y: 1 }, drawingCtx, { color: '#00F', lineWidth: 2 });
            }
            if (result.codeResult && result.codeResult.code) {
                Quagga.ImageDebug.drawPath(result.line, { x: 'x', y: 'y' }, drawingCtx, { color: 'red', lineWidth: 3 });
            }
        }
    }
}
NgxBarcodeScannerService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxBarcodeScannerService_Factory() { return new NgxBarcodeScannerService(); }, token: NgxBarcodeScannerService, providedIn: "root" });
NgxBarcodeScannerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxBarcodeScannerService.ctorParameters = () => [];

class Utils {
    static setOrDefault(object, path, value) {
        if (typeof object[path] === 'undefined') {
            object[path] = value;
        }
    }
}

class NgxBarcodeScannerComponent {
    constructor(service) {
        this.service = service;
        this.codes = [
            'code_128', 'ean', 'ean_8', 'code_39', 'code_39_vin',
            'codabar', 'upc', 'upc_e', 'i2of5', '2of5', 'code_93'
        ];
        this.valueChange = new EventEmitter();
        this.exception = new EventEmitter();
    }
    setConfig() {
        if (!this.config) {
            this.config = Object.assign(Object.assign({}, this.service.defaultConfig()), { decoder: {
                    readers: this.readers()
                } });
        }
        if (!this.config.inputStream) {
            this.config.inputStream = {};
        }
        Utils.setOrDefault(this.config.inputStream, 'name', 'Live');
        Utils.setOrDefault(this.config.inputStream, 'type', 'LiveStream');
        if (!this.config.locator) {
            this.config.locator = {};
        }
        Utils.setOrDefault(this.config.locator, 'patchSize', 'medium');
        Utils.setOrDefault(this.config.locator, 'halfSample', false);
        Utils.setOrDefault(this.config, 'locate', true);
        Utils.setOrDefault(this.config, 'numOfWorkers', 8);
        Utils.setOrDefault(this.config, 'frequency', 10);
        if (!this.config.decoder) {
            this.config.decoder = {};
        }
        Utils.setOrDefault(this.config.decoder, 'readers', this.readers());
    }
    ngOnInit() {
        this.setConfig();
        const threshold = isNaN(this.errorThreshold) ? 0.1 : this.errorThreshold;
        this.service.start(this.config, threshold).subscribe((value) => {
            this.valueChange.emit(value);
            this.service.stop();
        }, error => {
            this.exception.emit(error);
        });
    }
    ngOnDestroy() {
        this.service.stop();
    }
    readers() {
        const types = (typeof this.codes === 'string') ? [this.codes] : this.codes;
        return types.map(it => it + '_reader');
    }
}
NgxBarcodeScannerComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: 'ngx-barcode-scanner',
                template: "<div id=\"interactive\" class=\"viewport\"></div>\n",
                styles: ["::ng-deep #interactive.viewport{position:relative;width:100%;height:auto;overflow:hidden;text-align:center}::ng-deep #interactive.viewport>canvas,::ng-deep #interactive.viewport>video{max-width:100%;width:100%}::ng-deep canvas.drawing,::ng-deep canvas.drawingBuffer{position:absolute;left:0;top:0}::ng-deep video{width:100%;-o-object-fit:cover;object-fit:cover}:host{position:static;width:100%}"]
            },] }
];
NgxBarcodeScannerComponent.ctorParameters = () => [
    { type: NgxBarcodeScannerService }
];
NgxBarcodeScannerComponent.propDecorators = {
    codes: [{ type: Input }],
    config: [{ type: Input }],
    errorThreshold: [{ type: Input }],
    value: [{ type: Input }],
    valueChange: [{ type: Output }],
    exception: [{ type: Output }]
};

class NgxBarcodeScannerModule {
}
NgxBarcodeScannerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxBarcodeScannerComponent],
                imports: [],
                exports: [NgxBarcodeScannerComponent]
            },] }
];

/*
 * Public API Surface of ngx-barcode-scanner
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxBarcodeScannerComponent, NgxBarcodeScannerModule, NgxBarcodeScannerService };
//# sourceMappingURL=eisberg-labs-ngx-barcode-scanner.js.map
